@@ -34,8 +34,10 @@ namespace hud
 class Icon : public unity::IconTexture
 {
 public:
-  typedef nux::ObjectPtr<IconTexture> Ptr;
-  Icon(std::string const& icon_name, unsigned int size);
+  typedef nux::ObjectPtr<Icon> Ptr;
+  Icon();
+
+  void SetIcon(std::string const& icon_name, unsigned int icon_size, unsigned int tile_size, unsigned int padding);
 
 protected:
   void Draw(nux::GraphicsEngine& GfxContext, bool force_draw);
@@ -43,9 +45,6 @@ protected:
   std::string GetName() const;
 
 private:
-  nux::ObjectPtr<nux::BaseTexture> background_;
-  nux::ObjectPtr<nux::BaseTexture> gloss_;
-  nux::ObjectPtr<nux::BaseTexture> edge_;
   nux::ObjectPtr<HudIconTextureSource> icon_texture_source_;
   unity::ui::IconRenderer icon_renderer_;
 };
