@@ -99,6 +99,9 @@ LauncherIcon::LauncherIcon()
   for (int i = 0; i < max_num_monitors; ++i)
     _is_visible_on_monitor[i] = true;
 
+  tooltip_enabled = true;
+  tooltip_enabled.changed.connect(sigc::mem_fun(this, &LauncherIcon::OnTooltipEnabledChanged));
+
   tooltip_text.SetSetterFunction(sigc::mem_fun(this, &LauncherIcon::SetTooltipText));
   tooltip_text = "blank";
 
@@ -202,6 +205,7 @@ LauncherIcon::AddProperties(GVariantBuilder* builder)
   .add("icon_type", _icon_type)
   .add("tooltip_text", tooltip_text())
   .add("sort_priority", _sort_priority)
+  .add("shortcut", _shortcut)
   .add("monitors_visibility", g_variant_builder_end(&monitors_builder))
   .add("active", GetQuirk(QUIRK_ACTIVE))
   .add("visible", GetQuirk(QUIRK_VISIBLE))
@@ -480,6 +484,12 @@ bool LauncherIcon::SetTooltipText(std::string& target, std::string const& value)
   return result;
 }
 
+void LauncherIcon::OnTooltipEnabledChanged(bool value)
+{
+  if (!value)
+    HideTooltip();
+}
+
 void
 LauncherIcon::SetShortcut(guint64 shortcut)
 {
@@ -498,7 +508,7 @@ LauncherIcon::GetShortcut()
 void
 LauncherIcon::ShowTooltip()
 {
-  if (_quicklist && _quicklist->IsVisible())
+  if (!tooltip_enabled || (_quicklist && _quicklist->IsVisible()))
     return;
 
   int tip_x = 100;
@@ -514,6 +524,7 @@ LauncherIcon::ShowTooltip()
     LoadTooltip();
   _tooltip->ShowTooltipWithTipAt(tip_x, tip_y);
   _tooltip->ShowWindow(!tooltip_text().empty());
+  tooltip_visible.emit(_tooltip);
 }
 
 void
@@ -535,9 +546,10 @@ void LauncherIcon::RecvMouseLeave(int monitor)
 
   if (_tooltip)
     _tooltip->ShowWindow(false);
+  tooltip_visible.emit(nux::ObjectPtr<nux::View>(nullptr));
 }
 
-bool LauncherIcon::OpenQuicklist(bool default_to_first_item, int monitor)
+bool LauncherIcon::OpenQuicklist(bool select_first_item, int monitor)
 {
   std::list<DbusmenuMenuitem*> menus = Menus();
 
@@ -583,8 +595,8 @@ bool LauncherIcon::OpenQuicklist(bool default_to_first_item, int monitor)
     _quicklist->AddMenuItem(ql_item);
   }
 
-  if (default_to_first_item)
-    _quicklist->DefaultToFirstItem();
+  if (select_first_item)
+    _quicklist->SelectFirstItem();
 
   if (monitor < 0)
   {
@@ -620,13 +632,13 @@ bool LauncherIcon::OpenQuicklist(bool default_to_first_item, int monitor)
   return true;
 }
 
-void LauncherIcon::RecvMouseDown(int button, int monitor)
+void LauncherIcon::RecvMouseDown(int button, unsigned long key_flags, int monitor)
 {
   if (button == 3)
     OpenQuicklist();
 }
 
-void LauncherIcon::RecvMouseUp(int button, int monitor)
+void LauncherIcon::RecvMouseUp(int button, unsigned long key_flags, int monitor)
 {
   if (button == 3)
   {
@@ -635,14 +647,17 @@ void LauncherIcon::RecvMouseUp(int button, int monitor)
   }
 }
 
-void LauncherIcon::RecvMouseClick(int button, int monitor)
+void LauncherIcon::RecvMouseClick(int button, unsigned long key_flags, int monitor)
 {
   ActionArg arg(ActionArg::LAUNCHER, button);
   arg.monitor = monitor;
+  bool shift_pressed = nux::GetKeyModifierState(key_flags, nux::NUX_STATE_SHIFT);
 
-  if (button == 1)
+  // Click without shift
+  if (button == 1 && !shift_pressed)
     Activate(arg);
-  else if (button == 2)
+  // Middle click or click with shift
+  else if ((button == 2) || (button == 1 && shift_pressed))
     OpenInstance(arg);
 }
 
@@ -650,6 +665,7 @@ void LauncherIcon::HideTooltip()
 {
   if (_tooltip)
     _tooltip->ShowWindow(false);
+  tooltip_visible.emit(nux::ObjectPtr<nux::View>(nullptr));
 }
 
 gboolean
@@ -853,6 +869,11 @@ LauncherIcon::SetQuirk(LauncherIcon::Quirk quirk, bool value)
     UBusServer* ubus = ubus_server_get_default();
     ubus_server_send_message(ubus, UBUS_LAUNCHER_ICON_URGENT_CHANGED, g_variant_new_boolean(value));
   }
+
+  if (quirk == QUIRK_VISIBLE)
+  {
+     visibility_changed.emit();
+  }
 }
 
 gboolean
