@@ -1083,7 +1083,7 @@ void PanelMenuView::OnActiveWindowChanged(BamfMatcher *matcher,
       // if we've just started tracking this window and it is maximized, let's
       // make sure it's undecorated just in case it slipped by us earlier
       // (I'm looking at you, Chromium!)
-      if (_is_maximized && wm->IsWindowDecorated(xid))
+      if (_is_maximized && wm->HasWindowDecorations(xid))
       {
         wm->Undecorate(xid);
         _maximized_set.insert(xid);
@@ -1222,7 +1222,7 @@ void PanelMenuView::OnWindowMaximized(guint xid)
   }
 
   // update the state of the window in the _decor_map
-  _decor_map[xid] = WindowManager::Default()->IsWindowDecorated(xid);
+  _decor_map[xid] = WindowManager::Default()->HasWindowDecorations(xid);
 
   if (_decor_map[xid])
     WindowManager::Default()->Undecorate(xid);
@@ -1716,7 +1716,7 @@ void PanelMenuView::SetMonitor(int monitor)
     {
       Window xid = bamf_window_get_xid(window);
 
-      _decor_map[xid] = WindowManager::Default()->IsWindowDecorated(xid);
+      _decor_map[xid] = WindowManager::Default()->HasWindowDecorations(xid);
 
       if (_decor_map[xid])
         WindowManager::Default()->Undecorate(xid);
