@@ -1,6 +1,6 @@
 // -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright (C) 2010 Canonical Ltd
+ * Copyright (C) 2010-2012 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -42,7 +42,6 @@ PluginAdapter* PluginAdapter::_default = 0;
 #define MWM_HINTS_FUNCTIONS     (1L << 0)
 #define MWM_HINTS_DECORATIONS   (1L << 1)
 #define MWM_HINTS_UNDECORATED_UNITY 0x80
-#define _XA_MOTIF_WM_HINTS    "_MOTIF_WM_HINTS"
 
 /* static */
 PluginAdapter*
@@ -151,13 +150,17 @@ PluginAdapter::NotifyStateChange(CompWindow* window, unsigned int state, unsigne
 void
 PluginAdapter::NotifyNewDecorationState(guint32 xid)
 {
-  bool wasTracked = (_window_decoration_state.find (xid) != _window_decoration_state.end ());
+  auto deco_state_it = _window_decoration_state.find(xid);
+  bool wasTracked = (deco_state_it != _window_decoration_state.end());
   bool wasDecorated = false;
 
   if (wasTracked)
+  {
     wasDecorated = _window_decoration_state[xid];
+    _window_decoration_state.erase(deco_state_it);
+  }
 
-  bool decorated = IsWindowDecorated (xid);
+  bool decorated = HasWindowDecorations(xid);
 
   if (decorated == wasDecorated)
     return;
@@ -415,44 +418,74 @@ PluginAdapter::IsWindowMaximized(guint xid)
   return false;
 }
 
-bool
-PluginAdapter::IsWindowDecorated(guint32 xid)
+unsigned long
+PluginAdapter::GetMwnDecorations(Window window_id) const
 {
   Display* display = m_Screen->dpy();
-  Window win = xid;
-  Atom hints_atom = None;
   MotifWmHints* hints = NULL;
   Atom type = None;
   gint format;
   gulong nitems;
   gulong bytes_after;
-  bool ret = true;
+  unsigned long decorations = 0;
 
-  hints_atom = XInternAtom(display, _XA_MOTIF_WM_HINTS, false);
-
-  if (XGetWindowProperty(display, win, hints_atom, 0,
+  if (XGetWindowProperty(display, window_id, Atoms::mwmHints, 0,
                          sizeof(MotifWmHints) / sizeof(long), False,
-                         hints_atom, &type, &format, &nitems, &bytes_after,
+                         Atoms::mwmHints, &type, &format, &nitems, &bytes_after,
                          (guchar**)&hints) != Success)
-    return false;
+  {
+    return decorations;
+  }
+
+  decorations |= (MwmDecorAll | MwmDecorTitle);
 
   if (!hints)
-    return ret;
+    return decorations;
 
   /* Check for the presence of the high bit
    * if present, it means that we undecorated
    * this window, so don't mark it as undecorated */
-  if (type == hints_atom && format != 0 &&
-      hints->flags & MWM_HINTS_DECORATIONS)
+  if (type == Atoms::mwmHints && format != 0 && hints->flags & MWM_HINTS_DECORATIONS)
   {
-    /* Must have both bits set */
-    _window_decoration_state[xid] = ret =
-          (hints->decorations & (MwmDecorAll | MwmDecorTitle))  ||
-          (hints->decorations & MWM_HINTS_UNDECORATED_UNITY);
+    decorations = hints->decorations;
   }
 
   XFree(hints);
-  return ret;
+  return decorations;
+}
+
+bool
+PluginAdapter::HasWindowDecorations(guint32 xid) const
+{
+  Window window_id = xid;
+  auto deco_state_it = _window_decoration_state.find(window_id);
+
+  if (deco_state_it != _window_decoration_state.end())
+    return deco_state_it->second;
+
+  unsigned long decorations = GetMwnDecorations(window_id);
+
+  /* Must have both bits set */
+  bool decorated = (decorations & (MwmDecorAll | MwmDecorTitle)) ||
+                   (decorations & MWM_HINTS_UNDECORATED_UNITY);
+
+  _window_decoration_state[window_id] = decorated;
+  return decorated;
+}
+
+bool
+PluginAdapter::IsWindowDecorated(guint32 xid)
+{
+  Window window_id = xid;
+
+  bool decorated = GetMwnDecorations(window_id) & (MwmDecorAll | MwmDecorTitle);
+
+  if (decorated && GetCardinalProperty(window_id, Atoms::frameExtents).empty())
+  {
+    decorated = false;
+  }
+
+  return decorated;
 }
 
 bool
@@ -1023,7 +1056,6 @@ void
 PluginAdapter::SetMwmWindowHints(Window xid, MotifWmHints* new_hints)
 {
   Display* display = m_Screen->dpy();
-  Atom hints_atom = None;
   MotifWmHints* data = NULL;
   MotifWmHints* hints = NULL;
   Atom type = None;
@@ -1031,18 +1063,16 @@ PluginAdapter::SetMwmWindowHints(Window xid, MotifWmHints* new_hints)
   gulong nitems;
   gulong bytes_after;
 
-  hints_atom = XInternAtom(display, _XA_MOTIF_WM_HINTS, false);
-
   if (XGetWindowProperty(display,
                          xid,
-                         hints_atom, 0, sizeof(MotifWmHints) / sizeof(long),
+                         Atoms::mwmHints, 0, sizeof(MotifWmHints) / sizeof(long),
                          False, AnyPropertyType, &type, &format, &nitems,
                          &bytes_after, (guchar**)&data) != Success)
   {
     return;
   }
 
-  if (type != hints_atom || !data)
+  if (type != Atoms::mwmHints || !data)
   {
     hints = new_hints;
   }
@@ -1064,7 +1094,7 @@ PluginAdapter::SetMwmWindowHints(Window xid, MotifWmHints* new_hints)
 
   XChangeProperty(display,
                   xid,
-                  hints_atom, hints_atom, 32, PropModeReplace,
+                  Atoms::mwmHints, Atoms::mwmHints, 32, PropModeReplace,
                   (guchar*)hints, sizeof(MotifWmHints) / sizeof(long));
 
   if (data)
@@ -1074,6 +1104,9 @@ PluginAdapter::SetMwmWindowHints(Window xid, MotifWmHints* new_hints)
 void
 PluginAdapter::Decorate(guint32 xid)
 {
+  if (!HasWindowDecorations(xid))
+    return;
+
   MotifWmHints hints = { 0 };
 
   hints.flags = MWM_HINTS_DECORATIONS;
@@ -1085,6 +1118,9 @@ PluginAdapter::Decorate(guint32 xid)
 void
 PluginAdapter::Undecorate(guint32 xid)
 {
+  if (!IsWindowDecorated(xid))
+    return;
+
   MotifWmHints hints = { 0 };
 
   /* Set the high bit to indicate that we undecorated this
@@ -1319,3 +1355,29 @@ PluginAdapter::AddProperties(GVariantBuilder* builder)
          .add("expo_active", IsExpoActive())
          .add("viewport_switch_running", IsViewPortSwitchStarted());
 }
+
+std::vector<long> PluginAdapter::GetCardinalProperty(Window window_id, Atom atom) const
+{
+  Atom type;
+  int result, format;
+  unsigned long n_items, bytes_after;
+  long *buf = nullptr;
+
+  result = XGetWindowProperty(m_Screen->dpy(), window_id, atom, 0L, 65536, False,
+                              XA_CARDINAL, &type, &format, &n_items, &bytes_after,
+                              reinterpret_cast<unsigned char **>(&buf));
+
+  std::unique_ptr<long[], int(*)(void*)> buffer(buf, XFree);
+
+  if (result == Success && type == XA_CARDINAL && format == 32 && buffer && n_items > 0)
+  {
+    std::vector<long> values(n_items);
+
+    for (unsigned i = 0; i < n_items; ++i)
+      values[i] = buffer[i];
+
+    return values;
+  }
+
+  return std::vector<long>();
+}
