@@ -1,6 +1,6 @@
 // -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright (C) 2010 Canonical Ltd
+ * Copyright (C) 2010-2012 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -125,6 +125,7 @@ public:
   bool IsWindowClosable(guint32 xid);
   bool IsWindowMinimizable(guint32 xid);
   bool IsWindowMaximizable(guint32 xid);
+  bool HasWindowDecorations(guint32 xid) const;
 
   void Restore(guint32 xid);
   void RestoreAt(guint32 xid, int x, int y);
@@ -174,6 +175,9 @@ private:
 
   bool CheckWindowIntersection(nux::Geometry const& region, CompWindow* window);
   void SetMwmWindowHints(Window xid, MotifWmHints* new_hints);
+  unsigned long GetMwnDecorations(Window xid) const;
+
+  std::vector<long> GetCardinalProperty(Window xid, Atom atom) const;
 
   CompScreen* m_Screen;
   MultiActionList m_ExpoActionList;
@@ -193,7 +197,7 @@ private:
   bool _in_show_desktop;
   CompWindow* _last_focused_window;
 
-  std::map<guint32, unsigned int> _window_decoration_state;
+  mutable std::map<guint32, unsigned int> _window_decoration_state;
 
   static PluginAdapter* _default;
 };
