@@ -65,11 +65,9 @@ QuicklistView::QuicklistView()
   , _corner_radius(4)
   , _padding(13)
   , _left_padding_correction(-1)
-  , _bottom_padding_correction_normal(-2)
-  , _bottom_padding_correction_single_item(-4)
   , _offset_correction(-1)
   , _cairo_text_has_changed(true)
-  , _current_item_index(0)
+  , _current_item_index(-1)
 {
   SetGeometry(nux::Geometry(0, 0, 1, 1));
 
@@ -101,8 +99,6 @@ QuicklistView::QuicklistView()
   _item_layout     = new nux::VLayout(TEXT(""), NUX_TRACKER_LOCATION);
   _vlayout->AddLayout(_item_layout, 0);
 
-  _default_item_layout = new nux::VLayout(TEXT(""), NUX_TRACKER_LOCATION);
-  _vlayout->AddLayout(_default_item_layout, 0);
   _vlayout->AddLayout(_bottom_space, 0);
   _vlayout->SetMinimumWidth(140);
 
@@ -138,13 +134,35 @@ QuicklistView::RecvEndFocus()
 {
 }
 
-bool
-QuicklistView::IsMenuItemSeperator(int index)
+void
+QuicklistView::SelectItem(int index)
 {
-  QuicklistMenuItem* menu_item = NULL;
-  DbusmenuMenuitem*  item   = NULL;
-  const gchar*       label  = NULL;
-  bool               result = false;
+  CancelItemsPrelightStatus();
+  int target_item = -1;
+
+  if (IsMenuItemSelectable(index))
+  {
+    QuicklistMenuItem* menu_item = GetNthItems(index);
+
+    if (menu_item)
+    {
+      target_item = index;
+      menu_item->_prelight = true;
+    }
+  }
+
+  if (_current_item_index != target_item)
+  {
+    _current_item_index = target_item;
+    selection_change.emit();
+    QueueDraw();
+  }
+}
+
+bool
+QuicklistView::IsMenuItemSelectable(int index)
+{
+  QuicklistMenuItem* menu_item = nullptr;
 
   if (index < 0)
     return false;
@@ -153,17 +171,7 @@ QuicklistView::IsMenuItemSeperator(int index)
   if (!menu_item)
     return false;
 
-  item = menu_item->_menuItem;
-  if (!item)
-    return false;
-
-  label = dbusmenu_menuitem_property_get(item, DBUSMENU_MENUITEM_PROP_LABEL);
-  if (!label)
-    result = true;
-  else
-    result = false;
-
-  return result;
+  return menu_item->GetSelectable();
 }
 
 void
@@ -175,56 +183,103 @@ QuicklistView::RecvKeyPressed(unsigned long    eventType,
 {
   switch (key_sym)
   {
+      // home or page up (highlight the first menu-hitem)
+    case NUX_VK_PAGE_UP:
+    case NUX_VK_HOME:
+    {
+      int num_items = GetNumItems();
+      int target_index = -1;
+
+      do
+      {
+        ++target_index;
+      }
+      while (!IsMenuItemSelectable(target_index) && target_index < num_items);
+
+      if (target_index < num_items)
+        SelectItem(target_index);
+
+      break;
+    }
+      // end or page down (highlight the last menu-hitem)
+    case NUX_VK_PAGE_DOWN:
+    case NUX_VK_END:
+    {
+      int target_index = GetNumItems();
+
+      do
+      {
+        --target_index;
+      }
+      while (!IsMenuItemSelectable(target_index) && target_index >= 0);
+
+      if (target_index >= 0)
+        SelectItem(target_index);
+
+      break;
+    }
       // up (highlight previous menu-item)
     case NUX_VK_UP:
     case NUX_KP_UP:
-      // protect against edge-case of first item being a separator
-      if (_current_item_index == 1 && IsMenuItemSeperator(0))
-        break;
+    {
+      int target_index = _current_item_index;
+      bool loop_back = false;
 
-      if (_current_item_index > 0)
+      if (target_index <= 0)
+        target_index = GetNumItems();
+
+      do
       {
-        GetNthItems(_current_item_index)->_prelight = false;
-        _current_item_index--;
+        --target_index;
 
-        while (IsMenuItemSeperator(_current_item_index))
-          _current_item_index--;
-
-        selection_change.emit();
-
-        GetNthItems(_current_item_index)->_prelight = true;
-        QueueDraw();
+        // If the first item is not selectable, we must loop from the last one
+        if (!loop_back && target_index == 0 && !IsMenuItemSelectable(target_index))
+        {
+          loop_back = true;
+          target_index = GetNumItems() - 1;
+        }
       }
+      while (!IsMenuItemSelectable(target_index) && target_index >= 0);
+
+      if (target_index >= 0)
+        SelectItem(target_index);
+
       break;
+    }
 
       // down (highlight next menu-item)
     case NUX_VK_DOWN:
     case NUX_KP_DOWN:
-      // protect against edge-case of last item being a separator
-      if (_current_item_index == (GetNumItems() - 1) && IsMenuItemSeperator(GetNumItems()))
-        break;
+    {
+      int target_index = _current_item_index;
+      int num_items = GetNumItems();
+      bool loop_back = false;
 
-      if (_current_item_index < GetNumItems() - 1)
+      if (target_index >= num_items - 1)
+        target_index = -1;
+
+      do
       {
-        GetNthItems(_current_item_index)->_prelight = false;
-        _current_item_index++;
+        ++target_index;
 
-        while (IsMenuItemSeperator(_current_item_index))
-          _current_item_index++;
-
-        selection_change.emit();
-
-        GetNthItems(_current_item_index)->_prelight = true;
-        QueueDraw();
+        // If the last item is not selectable, we must loop from the first one
+        if (!loop_back && target_index == num_items - 1 && !IsMenuItemSelectable(target_index))
+        {
+          loop_back = true;
+          target_index = 0;
+        }
       }
+      while (!IsMenuItemSelectable(target_index) && target_index < num_items);
+
+      if (target_index < num_items)
+        SelectItem(target_index);
+
       break;
+    }
 
       // left (close quicklist, go back to laucher key-nav)
     case NUX_VK_LEFT:
     case NUX_KP_LEFT:
-      _current_item_index = 0;
-      selection_change.emit();
-      GetNthItems(_current_item_index)->_prelight = true;
       Hide();
       // inform Launcher we switch back to Launcher key-nav
       ubus_server_send_message(ubus_server_get_default(),
@@ -234,32 +289,23 @@ QuicklistView::RecvKeyPressed(unsigned long    eventType,
 
       // esc (close quicklist, exit key-nav)
     case NUX_VK_ESCAPE:
-      _current_item_index = 0;
-      selection_change.emit();
-      GetNthItems(_current_item_index)->_prelight = true;
       Hide();
       // inform UnityScreen we leave key-nav completely
       ubus_server_send_message(ubus_server_get_default(),
                                UBUS_LAUNCHER_END_KEY_NAV,
                                NULL);
-      selection_change.emit();
       break;
 
       // <SPACE>, <RETURN> (activate selected menu-item)
     case NUX_VK_SPACE:
     case NUX_VK_ENTER:
     case NUX_KP_ENTER:
-      if (_current_item_index >= 0 && _current_item_index < GetNumItems() &&
-          GetNthItems(_current_item_index)->GetEnabled())
+      if (IsMenuItemSelectable(_current_item_index))
       {
-
         dbusmenu_menuitem_handle_event(GetNthItems(_current_item_index)->_menuItem,
                                        "clicked",
                                        NULL,
                                        0);
-        _current_item_index = 0;
-        selection_change.emit();
-        GetNthItems(_current_item_index)->_prelight = true;
         Hide();
       }
       break;
@@ -279,14 +325,6 @@ QuicklistView::~QuicklistView()
     (*it)->UnReference();
   }
 
-  for (it = _default_item_list.begin(); it != _default_item_list.end(); it++)
-  {
-    // Remove from introspection
-    RemoveChild(*it);
-    (*it)->UnReference();
-  }
-
-  _default_item_list.clear();
   _item_list.clear();
 }
 
@@ -303,7 +341,7 @@ void QuicklistView::ShowQuicklistWithTipAt(int anchor_tip_x, int anchor_tip_y)
 
   if (!_enable_quicklist_for_testing)
   {
-    if ((_item_list.size() != 0) || (_default_item_list.size() != 0))
+    if ((_item_list.size() != 0))
     {
       int offscreen_size = GetBaseY() +
                            GetBaseHeight() -
@@ -365,6 +403,12 @@ void QuicklistView::Hide()
     UnGrabKeyboard();
     //EnableInputWindow (false);
     ShowWindow(false);
+
+    if (_current_item_index != -1)
+    {
+      selection_change.emit();
+      _current_item_index = -1;
+    }
   }
 }
 
@@ -385,12 +429,6 @@ void QuicklistView::Draw(nux::GraphicsEngine& gfxContext, bool forceDraw)
       (*it)->ProcessDraw(gfxContext, forceDraw);
   }
 
-  for (it = _default_item_list.begin(); it != _default_item_list.end(); it++)
-  {
-    if ((*it)->GetVisible())
-      (*it)->ProcessDraw(gfxContext, forceDraw);
-  }
-
   gfxContext.PopClippingRectangle();
 }
 
@@ -419,45 +457,34 @@ void QuicklistView::PreLayoutManagement()
     (*it)->GetTextExtents(textWidth, textHeight);
     if (textWidth > MaxItemWidth)
       MaxItemWidth = textWidth;
-    TotalItemHeight += textHeight;
-  }
-
-  for (it = _default_item_list.begin(); it != _default_item_list.end(); it++)
-  {
-    // Make sure item is in layout if it should be
-    if (!(*it)->GetVisible())
-    {
-      _default_item_layout->RemoveChildObject(*it);
-      continue;
-    }
-    else if (!(*it)->GetParentObject())
-      _default_item_layout->AddView(*it, 1, nux::eCenter, nux::eFull);
-
-    int  textWidth  = 0;
-    int  textHeight = 0;
-    (*it)->GetTextExtents(textWidth, textHeight);
-    if (textWidth > MaxItemWidth)
-      MaxItemWidth = textWidth;
+    textHeight += QuicklistMenuItem::ITEM_MARGIN * 2;
     TotalItemHeight += textHeight;
   }
 
   if (TotalItemHeight < _anchor_height)
   {
-    _top_space->SetMinMaxSize(1, (_anchor_height - TotalItemHeight) / 2 + 1 + _padding + _corner_radius);
-    _bottom_space->SetMinMaxSize(1, (_anchor_height - TotalItemHeight) / 2 + 1 +
-                                     _padding + _corner_radius +
-                                     _bottom_padding_correction_single_item);
+    int b = (_anchor_height - TotalItemHeight) / 2 + _padding + _corner_radius;
+    int t = b + _offset_correction;
+
+    _top_space->SetMinimumHeight(t);
+    _top_space->SetMaximumHeight(t);
+
+    _bottom_space->SetMinimumHeight(b);
+    _bottom_space->SetMaximumHeight(b);
   }
   else
   {
-    _top_space->SetMinMaxSize(_padding + _corner_radius, _padding + _corner_radius);
-    _bottom_space->SetMinMaxSize(_padding + _corner_radius - 2,
-                                 _padding + _corner_radius +
-                                 _bottom_padding_correction_normal);
+    int b = _padding + _corner_radius;
+    int t = b + _offset_correction;
+
+    _top_space->SetMinimumHeight(t);
+    _top_space->SetMaximumHeight(t);
+
+    _bottom_space->SetMinimumHeight(b);
+    _bottom_space->SetMaximumHeight(b);
   }
 
   _item_layout->SetMinimumWidth(MaxItemWidth);
-  _default_item_layout->SetMinimumWidth(MaxItemWidth);
 
   BaseWindow::PreLayoutManagement();
 }
@@ -469,7 +496,7 @@ long QuicklistView::PostLayoutManagement(long LayoutResult)
   UpdateTexture();
 
   int x = _padding + _anchor_width + _corner_radius + _offset_correction;
-  int y = _padding + _corner_radius + _offset_correction;
+  int y = _top_space->GetMinimumHeight();
 
   std::list<QuicklistMenuItem*>::iterator it;
   for (it = _item_list.begin(); it != _item_list.end(); it++)
@@ -483,23 +510,12 @@ long QuicklistView::PostLayoutManagement(long LayoutResult)
     y += (*it)->GetBaseHeight();
   }
 
-  for (it = _default_item_list.begin(); it != _default_item_list.end(); it++)
-  {
-    if (!(*it)->GetVisible())
-      continue;
-
-    (*it)->SetBaseX(x);
-    (*it)->SetBaseY(y);
-
-    y += (*it)->GetBaseHeight();
-  }
-
   // We must correct the width of line separators. The rendering of the separator can be smaller than the width of the
   // quicklist. The reason for that is, the quicklist width is determined by the largest entry it contains. That size is
   // only after MaxItemWidth is computed in QuicklistView::PreLayoutManagement.
   // The setting of the separator width is done here after the Layout cycle for this widget is over. The width of the separator
   // has bee set correctly during the layout cycle, but the cairo rendering still need to be adjusted.
-  int separator_width = nux::Max<int>(_default_item_layout->GetBaseWidth(), _item_layout->GetBaseWidth());
+  int separator_width = _item_layout->GetBaseWidth();
 
   for (it = _item_list.begin(); it != _item_list.end(); it++)
   {
@@ -511,16 +527,6 @@ long QuicklistView::PostLayoutManagement(long LayoutResult)
     }
   }
 
-  for (it = _default_item_list.begin(); it != _default_item_list.end(); it++)
-  {
-    QuicklistMenuItem* item = (QuicklistMenuItem*)(*it);
-    if (item->GetVisible() && item->CairoSurfaceWidth() != separator_width)
-    {
-      // Compute textures of the item.
-      item->UpdateTexture();
-    }
-  }
-
   return result;
 }
 
@@ -567,24 +573,6 @@ void QuicklistView::CheckAndEmitItemSignal(int x, int y)
       }
     }
   }
-
-  for (it = _default_item_list.begin(); it != _default_item_list.end(); it++)
-  {
-    if (!(*it)->GetVisible())
-      continue;
-
-    geo = (*it)->GetGeometry();
-    geo.width = _default_item_layout->GetBaseWidth();
-
-    if (geo.IsPointInside(x, y))
-    {
-      // An action is performed: send the signal back to the application
-      if ((*it)->_menuItem)
-      {
-        dbusmenu_menuitem_handle_event((*it)->_menuItem, "clicked", NULL, 0);
-      }
-    }
-  }
 }
 
 void QuicklistView::RecvItemMouseRelease(QuicklistMenuItem* item, int x, int y)
@@ -608,64 +596,42 @@ void QuicklistView::CancelItemsPrelightStatus()
   {
     (*it)->_prelight = false;
   }
-
-  for (it = _default_item_list.begin(); it != _default_item_list.end(); it++)
-  {
-    (*it)->_prelight = false;
-  }
 }
 
 void QuicklistView::RecvItemMouseDrag(QuicklistMenuItem* item, int x, int y)
 {
   nux::Geometry geo;
-  std::list<QuicklistMenuItem*>::iterator it;
-  for (it = _item_list.begin(); it != _item_list.end(); it++)
+
+  for (auto it : _item_list)
   {
-    if (!(*it)->GetVisible())
+    int item_index = GetItemIndex(it);
+
+    if (!IsMenuItemSelectable(item_index))
       continue;
 
-    geo = (*it)->GetGeometry();
+    geo = it->GetGeometry();
     geo.width = _item_layout->GetBaseWidth();
 
     if (geo.IsPointInside(x + item->GetBaseX(), y + item->GetBaseY()))
     {
-      (*it)->_prelight = true;
-    }
-    else
-    {
-      (*it)->_prelight = false;
+      SelectItem(item_index);
     }
   }
-
-  for (it = _default_item_list.begin(); it != _default_item_list.end(); it++)
-  {
-    if (!(*it)->GetVisible())
-      continue;
-
-    geo = (*it)->GetGeometry();
-    geo.width = _default_item_layout->GetBaseWidth();
-
-    if (geo.IsPointInside(x + item->GetBaseX(), y + item->GetBaseY()))
-    {
-      (*it)->_prelight = true;
-    }
-    else
-    {
-      (*it)->_prelight = false;
-    }
-  }
-
-  NeedRedraw();
 }
 
 void QuicklistView::RecvItemMouseEnter(QuicklistMenuItem* item)
 {
-  NeedRedraw();
+  int item_index = GetItemIndex(item);
+
+  SelectItem(item_index);
 }
 
 void QuicklistView::RecvItemMouseLeave(QuicklistMenuItem* item)
 {
-  NeedRedraw();
+  int item_index = GetItemIndex(item);
+
+  if (item_index < 0 || item_index == _current_item_index)
+    SelectItem(-1);
 }
 
 void QuicklistView::RecvMouseDown(int x, int y, unsigned long button_flags, unsigned long key_flags)
@@ -718,18 +684,9 @@ void QuicklistView::RemoveAllMenuItem()
     (*it)->UnReference();
   }
 
-  for (it = _default_item_list.begin(); it != _default_item_list.end(); it++)
-  {
-    // Remove from introspection
-    RemoveChild(*it);
-    (*it)->UnReference();
-  }
-
   _item_list.clear();
-  _default_item_list.clear();
 
   _item_layout->Clear();
-  _default_item_layout->Clear();
   _cairo_text_has_changed = true;
   nux::GetWindowThread()->QueueObjectLayout(this);
 }
@@ -764,7 +721,7 @@ void QuicklistView::RenderQuicklistView()
 
 int QuicklistView::GetNumItems()
 {
-  return _item_list.size() + _default_item_list.size();
+  return _item_list.size();
 }
 
 QuicklistMenuItem* QuicklistView::GetNthItems(int index)
@@ -780,20 +737,22 @@ QuicklistMenuItem* QuicklistView::GetNthItems(int index)
     }
   }
 
-  if (index < (int)_item_list.size() + (int)_default_item_list.size())
+  return nullptr;
+}
+
+int QuicklistView::GetItemIndex(QuicklistMenuItem* item)
+{
+  int index = -1;
+
+  for (auto it : _item_list)
   {
-    int i = 0;
-    if (_item_list.size() > 0)
-      i = _item_list.size() - 1;
-    std::list<QuicklistMenuItem*>::iterator it;
-    for (it = _item_list.begin(); it != _item_list.end(); i++, it++)
-    {
-      if (i == index)
-        return *it;
-    }
+    ++index;
+
+    if (it == item)
+      return index;
   }
 
-  return 0;
+  return index;
 }
 
 QuicklistMenuItemType QuicklistView::GetNthType(int index)
@@ -801,6 +760,7 @@ QuicklistMenuItemType QuicklistView::GetNthType(int index)
   QuicklistMenuItem* item = GetNthItems(index);
   if (item)
     return item->GetItemType();
+
   return MENUITEM_TYPE_UNKNOWN;
 }
 
@@ -809,13 +769,9 @@ std::list<QuicklistMenuItem*> QuicklistView::GetChildren()
   return _item_list;
 }
 
-void QuicklistView::DefaultToFirstItem()
+void QuicklistView::SelectFirstItem()
 {
-  if (GetNumItems() >= 1)
-  {
-    GetNthItems(0)->_prelight = true;
-    QueueDraw();
-  }
+  SelectItem(0);
 }
 
 void ql_tint_dot_hl(cairo_t* cr,
@@ -1229,7 +1185,7 @@ void QuicklistView::UpdateTexture()
 
   if (!_enable_quicklist_for_testing)
   {
-    if (!_item_list.empty() || !_default_item_list.empty())
+    if (!_item_list.empty())
     {
       int offscreen_size = GetBaseY() +
                            height -
@@ -1400,15 +1356,10 @@ std::string QuicklistView::GetName() const
 
 void QuicklistView::AddProperties(GVariantBuilder* builder)
 {
-  nux::Geometry abs_geo = GetAbsoluteGeometry();
-
   variant::BuilderWrapper(builder)
-    .add("x", abs_geo.x)
-    .add("y", abs_geo.y)
+    .add(GetAbsoluteGeometry())
     .add("base_x", GetBaseX())
     .add("base_y", GetBaseY())
-    .add("width", GetBaseWidth())
-    .add("height", GetBaseHeight())
     .add("active", IsVisible());
 }
 
