@@ -21,14 +21,10 @@
  */
 
 
-#include <sstream>     // for ostringstream
 #include "ResultRendererTile.h"
 
-#include <boost/algorithm/string.hpp>
-
 #include <pango/pango.h>
 #include <pango/pangocairo.h>
-#include <gdk/gdk.h>
 #include <gtk/gtk.h>
 
 #include <NuxCore/Logger.h>
@@ -51,6 +47,7 @@ namespace unity
 namespace
 {
 nux::logging::Logger logger("unity.dash.results");
+const std::string DEFAULT_GICON = ". GThemedIcon text-x-preview";
 
 const int FONT_SIZE = 10;
 }
@@ -230,7 +227,7 @@ nux::BaseTexture* ResultRendererTile::DrawHighlight(std::string const& texid, in
 
 void ResultRendererTile::Preload(Result& row)
 {
-  if (row.renderer<TextureContainer*>() == nullptr)
+  if (!row.renderer<TextureContainer*>())
   {
     row.set_renderer(new TextureContainer());
     LoadIcon(row);
@@ -240,17 +237,25 @@ void ResultRendererTile::Preload(Result& row)
 
 void ResultRendererTile::Unload(Result& row)
 {
-  TextureContainer *container = row.renderer<TextureContainer*>();
-  delete container;
+  delete row.renderer<TextureContainer*>();
   row.set_renderer<TextureContainer*>(nullptr);
 }
 
 void ResultRendererTile::LoadIcon(Result& row)
 {
+  auto container = row.renderer<TextureContainer*>();
+
+  if (!container)
+  {
+    LOG_ERROR(logger) << "No valid container for Result " << row.name() << " with URI "
+                      << row.uri();
+    return;
+  }
+
   Style& style = Style::Instance();
   std::string const& icon_hint = row.icon_hint;
-#define DEFAULT_GICON ". GThemedIcon text-x-preview"
   std::string icon_name;
+
   if (G_UNLIKELY(neko))
   {
     int tmp1 = style.GetTileIconSize() + (rand() % 16) - 8;
@@ -266,28 +271,25 @@ void ResultRendererTile::LoadIcon(Result& row)
     icon_name = !icon_hint.empty() ? icon_hint : DEFAULT_GICON;
   }
 
+  auto slot = sigc::bind(sigc::mem_fun(this, &ResultRendererTile::IconLoaded), icon_hint, row);
 
-
-  GIcon*  icon = g_icon_new_for_string(icon_name.c_str(), NULL);
-  TextureContainer* container = row.renderer<TextureContainer*>();
-
-  IconLoader::IconLoaderCallback slot = sigc::bind(sigc::mem_fun(this, &ResultRendererTile::IconLoaded), icon_hint, row);
-
-  if (g_strrstr(icon_name.c_str(), "://"))
+  if (icon_name.find("://") != std::string::npos)
   {
     container->slot_handle = IconLoader::GetDefault().LoadFromURI(icon_name, style.GetTileIconSize(), slot);
   }
-  else if (G_IS_ICON(icon))
-  {
-    container->slot_handle = IconLoader::GetDefault().LoadFromGIconString(icon_name, style.GetTileIconSize(), slot);
-  }
   else
   {
-    container->slot_handle = IconLoader::GetDefault().LoadFromIconName(icon_name, style.GetTileIconSize(), slot);
-  }
+    glib::Object<GIcon> icon(g_icon_new_for_string(icon_name.c_str(), nullptr));
 
-  if (icon != NULL)
-    g_object_unref(icon);
+    if (G_IS_ICON(icon.RawPtr()))
+    {
+      container->slot_handle = IconLoader::GetDefault().LoadFromGIconString(icon_name, style.GetTileIconSize(), slot);
+    }
+    else
+    {
+      container->slot_handle = IconLoader::GetDefault().LoadFromIconName(icon_name, style.GetTileIconSize(), slot);
+    }
+  }
 }
 
 nux::BaseTexture* ResultRendererTile::CreateTextureCallback(std::string const& texid,
