@@ -55,6 +55,7 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
   , preview_row_(0)
   , last_lazy_loaded_result_ (0)
   , lazy_load_handle_(0)
+  , view_changed_handle_(0)
   , last_mouse_down_x_(-1)
   , last_mouse_down_y_(-1)
   , recorded_dash_width_(-1)
@@ -69,9 +70,11 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
   {
     NeedRedraw();
   };
+
   horizontal_spacing.changed.connect(needredraw_lambda);
   vertical_spacing.changed.connect(needredraw_lambda);
   padding.changed.connect(needredraw_lambda);
+  selected_index_.changed.connect(needredraw_lambda);
 
   key_nav_focus_change.connect(sigc::mem_fun(this, &ResultViewGrid::OnKeyNavFocusChange));
   key_nav_focus_activate.connect([&] (nux::Area *area) { UriActivated.emit (focused_uri_); });
@@ -102,17 +105,21 @@ ResultViewGrid::ResultViewGrid(NUX_FILE_LINE_DECL)
   });
 
   SetDndEnabled(true, false);
-  NeedRedraw();
 }
 
 ResultViewGrid::~ResultViewGrid()
 {
-  g_source_remove(lazy_load_handle_);
+  if (lazy_load_handle_)
+    g_source_remove(lazy_load_handle_);
+
+  if (view_changed_handle_)
+    g_source_remove(view_changed_handle_);
 }
 
 gboolean ResultViewGrid::OnLazyLoad (gpointer data)
 {
   ResultViewGrid *self = (ResultViewGrid*)data;
+  self->lazy_load_handle_ = 0;
   self->DoLazyLoad();
   return FALSE;
 }
@@ -126,9 +133,26 @@ void ResultViewGrid::QueueLazyLoad()
   last_lazy_loaded_result_ = 0; // we always want to reset the lazy load index here
 }
 
+void ResultViewGrid::QueueViewChanged()
+{
+  if (view_changed_handle_ == 0)
+  {
+    // using G_PRIORITY_HIGH because this needs to happen *before* next draw
+    view_changed_handle_ = g_idle_add_full (G_PRIORITY_HIGH,
+                                            [](gpointer data) -> gboolean
+    {
+      ResultViewGrid *self = (ResultViewGrid*)data;
+      self->SizeReallocate();
+      self->last_lazy_loaded_result_ = 0; // reset the lazy load index
+      self->DoLazyLoad(); // also calls QueueDraw
+      self->view_changed_handle_ = 0;
+      return FALSE;
+    }, this, NULL);
+  }
+}
+
 void ResultViewGrid::DoLazyLoad()
 {
-  lazy_load_handle_ = 0;
   // FIXME - so this code was nice, it would only load the visible entries on the screen
   // however nux does not give us a good enough indicator right now that we are scrolling,
   // thus if you scroll more than a screen in one frame, you will end up with at least one frame where
@@ -190,7 +214,7 @@ void ResultViewGrid::DoLazyLoad()
 
 int ResultViewGrid::GetItemsPerRow()
 {
-  int items_per_row = (GetGeometry().width - (padding * 2)) / (renderer_->width + horizontal_spacing);
+  int items_per_row = (GetGeometry().width - (padding * 2) + horizontal_spacing) / (renderer_->width + horizontal_spacing);
   return (items_per_row) ? items_per_row : 1; // always at least one item per row
 }
 
@@ -235,17 +259,13 @@ void ResultViewGrid::SetModelRenderer(ResultRenderer* renderer)
 void ResultViewGrid::AddResult(Result& result)
 {
   results_.push_back(result);
-
-  SizeReallocate();
-  QueueLazyLoad();
-  NeedRedraw();
+  QueueViewChanged();
 }
 
 void ResultViewGrid::RemoveResult(Result& result)
 {
   ResultView::RemoveResult(result);
-  SizeReallocate();
-  QueueLazyLoad();
+  QueueViewChanged();
 }
 
 void ResultViewGrid::SizeReallocate()
@@ -267,7 +287,7 @@ void ResultViewGrid::SizeReallocate()
   }
   else
   {
-    total_height = renderer_->height;
+    total_height = renderer_->height + vertical_spacing;
   }
 
   int width = (items_per_row * renderer_->width) + (padding*2) + ((items_per_row - 1) * horizontal_spacing);
@@ -463,30 +483,30 @@ void ResultViewGrid::OnKeyDown (unsigned long event_type, unsigned long event_ke
   {
     case (nux::KEY_NAV_LEFT):
     {
-      --selected_index_;
+      selected_index_ = selected_index_ - 1;
       break;
     }
     case (nux::KEY_NAV_RIGHT):
     {
-      ++selected_index_;
+     selected_index_ = selected_index_ + 1;
       break;
     }
     case (nux::KEY_NAV_UP):
     {
-      selected_index_ -= items_per_row;
+      selected_index_ = selected_index_ - items_per_row;
       break;
     }
     case (nux::KEY_NAV_DOWN):
     {
-      selected_index_ += items_per_row;
+      selected_index_ = selected_index_ + items_per_row;
       break;
     }
     default:
       break;
   }
 
-  selected_index_ = std::max(0, selected_index_);
-  selected_index_ = std::min(static_cast<int>(results_.size() - 1), selected_index_);
+  selected_index_ = std::max(0, selected_index_());
+  selected_index_ = std::min(static_cast<int>(results_.size() - 1), selected_index_());
   focused_uri_ = results_[selected_index_].uri;
 
   int focused_x = (renderer_->width + horizontal_spacing + extra_horizontal_spacing_) * (selected_index_ % items_per_row);
@@ -495,8 +515,6 @@ void ResultViewGrid::OnKeyDown (unsigned long event_type, unsigned long event_ke
   ubus_.SendMessage(UBUS_RESULT_VIEW_KEYNAV_CHANGED,
                     g_variant_new("(iiii)", focused_x, focused_y, renderer_->width(), renderer_->height()));
   selection_change.emit();
-
-  NeedRedraw();
 }
 
 nux::Area* ResultViewGrid::KeyNavIteration(nux::KeyNavDirection direction)
@@ -508,7 +526,7 @@ void ResultViewGrid::OnKeyNavFocusChange(nux::Area *area, bool has_focus, nux::K
 {
   if (HasKeyFocus())
   {
-    if (selected_index_ < 0)
+    if (selected_index_ < 0 and !results_.empty())
     {
         focused_uri_ = results_.front().uri;
         selected_index_ = 0;
@@ -554,8 +572,6 @@ void ResultViewGrid::OnKeyNavFocusChange(nux::Area *area, bool has_focus, nux::K
 
     selection_change.emit();
   }
-
-  NeedRedraw();
 }
 
 long ResultViewGrid::ComputeContentSize()
@@ -564,7 +580,6 @@ long ResultViewGrid::ComputeContentSize()
   QueueLazyLoad();
   long ret = ResultView::ComputeContentSize();
   return ret;
-
 }
 
 
@@ -718,12 +733,11 @@ void ResultViewGrid::MouseMove(int x, int y, int dx, int dy, unsigned long butto
   if (mouse_over_index_ != index)
   {
     selected_index_ = mouse_over_index_ = index;
+
     nux::GetWindowCompositor().SetKeyFocusArea(this);
   }
   mouse_last_x_ = x;
   mouse_last_y_ = y;
-
-  NeedRedraw();
 }
 
 void ResultViewGrid::MouseClick(int x, int y, unsigned long button_flags, unsigned long key_flags)
