@@ -251,7 +251,7 @@ nux::Geometry SwitcherView::UpdateRenderTargets (nux::Point const& center, times
 {
   std::vector<Window> xids = model_->DetailXids ();
 
-  int ms_since_change = TimeUtil::TimeDelta(&current, &save_time_);
+  DeltaTime ms_since_change = TimeUtil::TimeDelta(&current, &save_time_);
   float progress = MIN (1.0f, (float) ms_since_change / (float) animation_length());
 
   for (Window window : xids)
@@ -508,7 +508,7 @@ std::list<RenderArg> SwitcherView::RenderArgsFlat(nux::Geometry& background_geo,
       ++i;
     }
 
-    int ms_since_change = TimeUtil::TimeDelta(&current, &save_time_);
+    DeltaTime ms_since_change = TimeUtil::TimeDelta(&current, &save_time_);
     if (saved_args_.size () == results.size () && ms_since_change < animation_length)
     {
       float progress = (float) ms_since_change / (float) animation_length();
@@ -630,7 +630,7 @@ void SwitcherView::DrawOverlay(nux::GraphicsEngine& GfxContext, bool force_draw,
   text_view_->SetBaseY(last_background_.y + last_background_.height - 45);
   text_view_->Draw(GfxContext, force_draw);
 
-  int ms_since_change = TimeUtil::TimeDelta(&current_, &save_time_);
+  DeltaTime ms_since_change = TimeUtil::TimeDelta(&current_, &save_time_);
 
   if (ms_since_change < animation_length && redraw_handle_ == 0)
     redraw_handle_ = g_idle_add_full (G_PRIORITY_DEFAULT, &SwitcherView::OnDrawTimeout, this, NULL);
